@@ -28,6 +28,7 @@ from solaar.ui import notify as _notify
 from . import hidpp10 as _hidpp10
 from . import hidpp20 as _hidpp20
 from . import special_keys as _special_keys
+from .common import NamedInt as _NamedInt
 from .common import NamedInts as _NamedInts
 from .common import bytes2int as _bytes2int
 from .common import int2bytes as _int2bytes
@@ -80,6 +81,7 @@ _HIRES_INV = ('hires-smooth-invert', _('Scroll Wheel Direction'),
               _('Invert direction for vertical scroll with wheel.'))
 _HIRES_RES = ('hires-smooth-resolution', _('Scroll Wheel Resolution'),
               _('High-sensitivity mode for vertical scroll with the wheel.'))
+_REPORT_RATE = ('report_rate', _('Polling Rate (ms)'), _('Frequency of device polling, in milliseconds'))
 _FN_SWAP = ('fn-swap', _('Swap Fx function'),
             _('When set, the F1..F12 keys will activate their special function,\n'
               'and you must hold the FN key to activate their standard function.') + '\n\n' +
@@ -95,6 +97,8 @@ _BACKLIGHT = ('backlight', _('Backlight'), _('Turn illumination on or off on key
 _REPROGRAMMABLE_KEYS = ('reprogrammable-keys', _('Key/Button Actions'),
                         _('Change the action for the key or button.') + '\n' +
                         _('Changing important actions (such as for the left mouse button) can result in an unusable system.'))
+_DIVERT_KEYS = ('divert-keys', _('Key/Button Diversion'),
+                _('Make the key or button send HID++ notifications (which trigger Solaar rules but are otherwise ignored).'))
 _DISABLE_KEYS = ('disable-keyboard-keys', _('Disable keys'), _('Disable specific keyboard keys.'))
 _PLATFORM = ('multiplatform', _('Set OS'), _('Change keys to match OS.'))
 _CHANGE_HOST = ('change-host', _('Change Host'), _('Switch connection to a different host'))
@@ -104,13 +108,17 @@ _THUMB_SCROLL_MODE = ('thumb-scroll-mode', _('Thumb Wheel HID++ Scrolling'),
 _THUMB_SCROLL_INVERT = ('thumb-scroll-invert', _('Thumb Wheel Direction'), _('Invert thumb wheel scroll direction.'))
 _GESTURE2_GESTURES = ('gesture2-gestures', _('Gestures'), _('Tweak the mouse/touchpad behaviour.'))
 _GESTURE2_PARAMS = ('gesture2-params', _('Gesture params'), _('Change numerical parameters of a mouse/touchpad.'))
-_DPI_SLIDING = ('dpi-sliding', _('DPI Sliding Ajustment'),
+_DPI_SLIDING = ('dpi-sliding', _('DPI Sliding Adjustment'),
                 _('Adjust the DPI by sliding the mouse horizontally while holding the DPI button.'))
+_DIVERT_CROWN = ('divert-crown', _('Divert crown events'),
+                 _('Make crown send CROWN HID++ notifications (which trigger Solaar rules but are otherwise ignored).'))
+_DIVERT_GKEYS = ('divert-gkeys', _('Divert G Keys'),
+                 _('Make G keys send GKEY HID++ notifications (which trigger Solaar rules but are otherwise ignored).'))
 
 _GESTURE2_GESTURES_LABELS = {
     _GG['Tap1Finger']: (_('Single tap'), _('Performs a left click.')),
-    _GG['Tap2Finger']: (_('Double tap'), _('Performs a right click.')),
-    _GG['Tap3Finger']: (_('Triple tap'), None),
+    _GG['Tap2Finger']: (_('Single tap with two fingers'), _('Performs a right click.')),
+    _GG['Tap3Finger']: (_('Single tap with three fingers'), None),
     _GG['Click1Finger']: (None, None),
     _GG['Click2Finger']: (None, None),
     _GG['Click3Finger']: (None, None),
@@ -520,6 +528,28 @@ def _feature_adjustable_dpi():
     return _Setting(_DPI, rw, callback=_feature_adjustable_dpi_callback, device_kind=(_DK.mouse, _DK.trackball))
 
 
+# Implemented based on code in libratrag
+def _feature_report_rate_callback(device):
+    # Host mode is required for report rate to be adjustable
+    if _hidpp20.get_onboard_mode(device) != _hidpp20.ONBOARD_MODES.MODE_HOST:
+        _hidpp20.set_onboard_mode(device, _hidpp20.ONBOARD_MODES.MODE_HOST)
+
+    reply = device.feature_request(_F.REPORT_RATE, 0x00)
+    assert reply, 'Oops, report rate choices cannot be retrieved!'
+    rate_list = []
+    rate_flags = _bytes2int(reply[0:1])
+    for i in range(0, 8):
+        if (rate_flags >> i) & 0x01:
+            rate_list.append(i + 1)
+    return _ChoicesV(_NamedInts.list(rate_list), byte_count=1) if rate_list else None
+
+
+def _feature_report_rate():
+    """Report Rate feature"""
+    rw = _FeatureRW(_F.REPORT_RATE, read_fnid=0x10, write_fnid=0x20)
+    return _Setting(_REPORT_RATE, rw, callback=_feature_report_rate_callback, device_kind=(_DK.mouse, ))
+
+
 def _feature_pointer_speed():
     """Pointer Speed feature"""
     # min and max values taken from usb traces of Win software
@@ -550,6 +580,21 @@ def _feature_reprogrammable_keys():
     return _Settings(_REPROGRAMMABLE_KEYS, rw, callback=_feature_reprogrammable_keys_callback, device_kind=(_DK.keyboard, ))
 
 
+def _feature_divert_keys_callback(device):
+    choices = {}
+    for k in device.keys:
+        if 'divertable' in k.flags:
+            choices[k.key] = [_NamedInt(0x00, 'Regular'), _NamedInt(0x01, 'Diverted')]
+    if not choices:
+        return None
+    return _ChoicesMapV(choices, key_byte_count=2, byte_count=1, mask=0x01, activate=0x02)
+
+
+def _feature_divert_keys():
+    rw = _FeatureRWMap(_F.REPROG_CONTROLS_V4, read_fnid=0x20, write_fnid=0x30, key_byte_count=2)
+    return _Settings(_DIVERT_KEYS, rw, callback=_feature_divert_keys_callback, device_kind=(_DK.keyboard, ))
+
+
 def _feature_disable_keyboard_keys_callback(device):
     mask = device.feature_request(_F.KEYBOARD_DISABLE_KEYS)[0]
     options = [_special_keys.DISABLE[1 << i] for i in range(8) if mask & (1 << i)]
@@ -684,6 +729,23 @@ def _feature_gesture2_params():
     return s
 
 
+def _feature_divert_crown():
+    rw = _FeatureRW(_F.CROWN, read_fnid=0x10, write_fnid=0x20)
+    return _Setting(_DIVERT_CROWN, rw, _BooleanV(true_value=0x02, false_value=0x01, mask=0xff), device_kind=(_DK.keyboard, ))
+
+
+def _feature_divert_gkeys():
+    class _DivertGkeysRW(_FeatureRW):
+        def __init__(self, feature):
+            super(_DivertGkeysRW, self).__init__(feature, write_fnid=0x20)
+
+        def read(self, device):  # no way to read, so just assume not diverted
+            return 0x00
+
+    rw = _DivertGkeysRW(_F.GKEY)
+    return _Setting(_DIVERT_GKEYS, rw, _BooleanV(true_value=0x01, false_value=0x00, mask=0xff), device_kind=(_DK.keyboard, ))
+
+
 #
 #
 #
@@ -707,13 +769,17 @@ _SETTINGS_TABLE = [
     _S(_THUMB_SCROLL_INVERT, _F.THUMB_WHEEL, _feature_thumb_invert),
     _S(_DPI, _F.ADJUSTABLE_DPI, _feature_adjustable_dpi, registerFn=_register_dpi),
     _S(_DPI_SLIDING, _F.REPROG_CONTROLS_V4, _feature_dpi_sliding),
+    _S(_POINTER_SPEED, _F.POINTER_SPEED, _feature_pointer_speed),
     _S(_BACKLIGHT, _F.BACKLIGHT2, _feature_backlight2),
     _S(_FN_SWAP, _F.FN_INVERSION, _feature_fn_swap, registerFn=_register_fn_swap),
     _S(_FN_SWAP, _F.NEW_FN_INVERSION, _feature_new_fn_swap, identifier='new_fn_swap'),
     _S(_FN_SWAP, _F.K375S_FN_INVERSION, _feature_k375s_fn_swap, identifier='k375s_fn_swap'),
-    _S(_POINTER_SPEED, _F.POINTER_SPEED, _feature_pointer_speed),
     _S(_REPROGRAMMABLE_KEYS, _F.REPROG_CONTROLS_V4, _feature_reprogrammable_keys),
+    _S(_DIVERT_KEYS, _F.REPROG_CONTROLS_V4, _feature_divert_keys),
     _S(_DISABLE_KEYS, _F.KEYBOARD_DISABLE_KEYS, _feature_disable_keyboard_keys),
+    _S(_REPORT_RATE, _F.REPORT_RATE, _feature_report_rate),
+    _S(_DIVERT_CROWN, _F.CROWN, _feature_divert_crown),
+    _S(_DIVERT_GKEYS, _F.GKEY, _feature_divert_gkeys),
     _S(_PLATFORM, _F.MULTIPLATFORM, _feature_multiplatform),
     _S(_PLATFORM, _F.DUALPLATFORM, _feature_dualplatform, identifier='dualplatform'),
     _S(_CHANGE_HOST, _F.CHANGE_HOST, _feature_change_host),
