@@ -34,162 +34,161 @@ CONTROL = _NamedInts(
     Previous=0x0006,
     Stop=0x0007,
     Application_Switcher=0x0008,
-    BURN=0x0009,
+    Burn=0x0009,
     Calculator=0x000A,
-    CALENDAR=0x000B,
-    CLOSE=0x000C,
-    EJECT=0x000D,
+    Calendar=0x000B,
+    Close=0x000C,
+    Eject=0x000D,
     Mail=0x000E,
-    HELP_AS_HID=0x000F,
-    HELP_AS_F1=0x0010,
-    LAUNCH_WORD_PROC=0x0011,
-    LAUNCH_SPREADSHEET=0x0012,
-    LAUNCH_PRESENTATION=0x0013,
-    UNDO_AS_CTRL_Z=0x0014,
-    UNDO_AS_HID=0x0015,
-    REDO_AS_CTRL_Y=0x0016,
-    REDO_AS_HID=0x0017,
-    PRINT_AS_CTRL_P=0x0018,
-    PRINT_AS_HID=0x0019,
-    SAVE_AS_CTRL_S=0x001A,
-    SAVE_AS_HID=0x001B,
-    PRESET_A=0x001C,
-    PRESET_B=0x001D,
-    PRESET_C=0x001E,
-    PRESET_D=0x001F,
-    FAVORITES=0x0020,
-    GADGETS=0x0021,
-    MY_HOME=0x0022,
-    GADGETS_AS_WIN_G=0x0023,
-    MAXIMIZE_AS_HID=0x0024,
-    MAXIMIZE_AS_WIN_SHIFT_M=0x0025,
-    MINIMIZE_AS_HID=0x0026,
-    MINIMIZE_AS_WIN_M=0x0027,
-    MEDIA_PLAYER=0x0028,
-    MEDIA_CENTER_LOGI=0x0029,
-    MEDIA_CENTER_MSFT=0x002A,  # Should not be used as it is not reprogrammable under Windows
-    CUSTOM_MENU=0x002B,
-    MESSENGER=0x002C,
-    MY_DOCUMENTS=0x002D,
-    MY_MUSIC=0x002E,
-    WEBCAM=0x002F,
-    MY_PICTURES=0x0030,
-    MY_VIDEOS=0x0031,
-    MY_COMPUTER_AS_HID=0x0032,
-    MY_COMPUTER_AS_WIN_E=0x0033,
-    LAUNC_PICTURE_VIEWER=0x0035,
-    ONE_TOUCH_SEARCH=0x0036,
-    PRESET_1=0x0037,
-    PRESET_2=0x0038,
-    PRESET_3=0x0039,
-    PRESET_4=0x003A,
-    RECORD=0x003B,
-    INTERNET_REFRESH=0x003C,
-    ROTATE_RIGHT=0x003D,
+    Help_As_HID=0x000F,
+    Help_As_F1=0x0010,
+    Launch_Word_Proc=0x0011,
+    Launch_Spreadsheet=0x0012,
+    Launch_Presentation=0x0013,
+    Undo_As_Ctrl_Z=0x0014,
+    Undo_As_HID=0x0015,
+    Redo_As_Ctrl_Y=0x0016,
+    Redo_As_HID=0x0017,
+    Print_As_Ctrl_P=0x0018,
+    Print_As_HID=0x0019,
+    Save_As_Ctrl_S=0x001A,
+    Save_As_HID=0x001B,
+    Preset_A=0x001C,
+    Preset_B=0x001D,
+    Preset_C=0x001E,
+    Preset_D=0x001F,
+    Favorites=0x0020,
+    Gadgets=0x0021,
+    My_Home=0x0022,
+    Gadgets_As_Win_G=0x0023,
+    Maximize_As_HID=0x0024,
+    Maximize_As_Win_Shift_M=0x0025,
+    Minimize_As_HID=0x0026,
+    Minimize_As_Win_M=0x0027,
+    Media_Player=0x0028,
+    Media_Center_Logi=0x0029,
+    Media_Center_Msft=0x002A,  # Should not be used as it is not reprogrammable under Windows
+    Custom_Menu=0x002B,
+    Messenger=0x002C,
+    My_Documents=0x002D,
+    My_Music=0x002E,
+    Webcam=0x002F,
+    My_Pictures=0x0030,
+    My_Videos=0x0031,
+    My_Computer_As_HID=0x0032,
+    My_Computer_As_Win_E=0x0033,
+    Launch_Picture_Viewer=0x0035,
+    One_Touch_Search=0x0036,
+    Preset_1=0x0037,
+    Preset_2=0x0038,
+    Preset_3=0x0039,
+    Preset_4=0x003A,
+    Record=0x003B,
+    Internet_Refresh=0x003C,
     Search=0x003E,  # SEARCH
-    SHUFFLE=0x003F,
-    SLEEP=0x0040,
-    INTERNET_STOP=0x0041,
-    SYNCHRONIZE=0x0042,
-    ZOOM=0x0043,
-    ZOOM_IN_AS_HID=0x0044,
-    ZOOM_IN_AS_CTRL_WHEEL=0x0045,
-    ZOOM_IN_AS_CLTR_PLUS=0x0046,
-    ZOOM_OUT_AS_HID=0x0047,
-    ZOOM_OUT_AS_CTRL_WHEEL=0x0048,
-    ZOOM_OUT_AS_CLTR_MINUS=0x0049,
-    ZOOM_RESET=0x004A,
-    ZOOM_FULL_SCREEN=0x004B,
-    PRINT_SCREEN=0x004C,
-    PAUSE_BREAK=0x004D,
-    SCROLL_LOCK=0x004E,
-    CONTEXTUAL_MENU=0x004F,
+    Shuffle=0x003F,
+    Sleep=0x0040,
+    Internet_Stop=0x0041,
+    Synchronize=0x0042,
+    Zoom=0x0043,
+    Zoom_In_As_HID=0x0044,
+    Zoom_In_As_Ctrl_Wheel=0x0045,
+    Zoom_In_As_Cltr_Plus=0x0046,
+    Zoom_Out_As_HID=0x0047,
+    Zoom_Out_As_Ctrl_Wheel=0x0048,
+    Zoom_Out_As_Ctrl_Minus=0x0049,
+    Zoom_Reset=0x004A,
+    Zoom_Full_Screen=0x004B,
+    Print_Screen=0x004C,
+    Pause_Break=0x004D,
+    Scroll_Lock=0x004E,
+    Contextual_Menu=0x004F,
     Left_Button=0x0050,  # LEFT_CLICK
     Right_Button=0x0051,  # RIGHT_CLICK
     Middle_Button=0x0052,  # MIDDLE_BUTTON
     Back_Button=0x0053,  # from M510v2 was BACK_AS_BUTTON_4
     Back=0x0054,  # BACK_AS_HID
-    BACK_AS_ALT_WIN_ARROW=0x0055,
+    Back_As_Alt_Win_Arrow=0x0055,
     Forward_Button=0x0056,  # from M510v2 was FORWARD_AS_BUTTON_5
-    FORWARD_AS_HID=0x0057,
-    FORWARD_AS_ALT_WIN_ARROW=0x0058,
-    BUTTON_6=0x0059,
-    LEFT_SCROLL_AS_BUTTON_7=0x005A,
+    Forward_As_HID=0x0057,
+    Forward_As_Alt_Win_Arrow=0x0058,
+    Button_6=0x0059,
+    Left_Scroll_As_Button_7=0x005A,
     Left_Tilt=0x005B,  # from M510v2 was LEFT_SCROLL_AS_AC_PAN
-    RIGHT_SCROLL_AS_BUTTON_8=0x005C,
+    Right_Scroll_As_Button_8=0x005C,
     Right_Tilt=0x005D,  # from M510v2 was RIGHT_SCROLL_AS_AC_PAN
-    BUTTON_9=0x005E,
-    BUTTON_10=0x005F,
-    BUTTON_11=0x0060,
-    BUTTON_12=0x0061,
-    BUTTON_13=0x0062,
-    BUTTON_14=0x0063,
-    BUTTON_15=0x0064,
-    BUTTON_16=0x0065,
-    BUTTON_17=0x0066,
-    BUTTON_18=0x0067,
-    BUTTON_19=0x0068,
-    BUTTON_20=0x0069,
-    BUTTON_21=0x006A,
-    BUTTON_22=0x006B,
-    BUTTON_23=0x006C,
-    BUTTON_24=0x006D,
+    Button_9=0x005E,
+    Button_10=0x005F,
+    Button_11=0x0060,
+    Button_12=0x0061,
+    Button_13=0x0062,
+    Button_14=0x0063,
+    Button_15=0x0064,
+    Button_16=0x0065,
+    Button_17=0x0066,
+    Button_18=0x0067,
+    Button_19=0x0068,
+    Button_20=0x0069,
+    Button_21=0x006A,
+    Button_22=0x006B,
+    Button_23=0x006C,
+    Button_24=0x006D,
     Show_Desktop=0x006E,  # Show_Desktop
     Lock_PC=0x006F,
-    FN_F1=0x0070,
-    FN_F2=0x0071,
-    FN_F3=0x0072,
-    FN_F4=0x0073,
-    FN_F5=0x0074,
-    FN_F6=0x0075,
-    FN_F7=0x0076,
-    FN_F8=0x0077,
-    FN_F9=0x0078,
-    FN_F10=0x0079,
-    FN_F11=0x007A,
-    FN_F12=0x007B,
-    FN_F13=0x007C,
-    FN_F14=0x007D,
-    FN_F15=0x007E,
-    FN_F16=0x007F,
-    FN_F17=0x0080,
-    FN_F18=0x0081,
-    FN_F19=0x0082,
-    IOS_HOME=0x0083,
-    ANDROID_HOME=0x0084,
-    ANDROID_MENU=0x0085,
-    ANDROID_SEARCH=0x0086,
-    ANDROID_BACK=0x0087,
-    HOME_COMBO=0x0088,
-    LOCK_COMBO=0x0089,
-    IOS_VIRTUAL_KEYBOARD=0x008A,
-    IOS_LANGUAGE_SWICH=0x008B,
-    MAC_EXPOSE=0x008C,
-    MAC_DASHBOARD=0x008D,
-    WIN7_SNAP_LEFT=0x008E,
-    WIN7_SNAP_RIGHT=0x008F,
+    Fn_F1=0x0070,
+    Fn_F2=0x0071,
+    Fn_F3=0x0072,
+    Fn_F4=0x0073,
+    Fn_F5=0x0074,
+    Fn_F6=0x0075,
+    Fn_F7=0x0076,
+    Fn_F8=0x0077,
+    Fn_F9=0x0078,
+    Fn_F10=0x0079,
+    Fn_F11=0x007A,
+    Fn_F12=0x007B,
+    Fn_F13=0x007C,
+    Fn_F14=0x007D,
+    Fn_F15=0x007E,
+    Fn_F16=0x007F,
+    Fn_F17=0x0080,
+    Fn_F18=0x0081,
+    Fn_F19=0x0082,
+    IOS_Home=0x0083,
+    Android_Home=0x0084,
+    Android_Menu=0x0085,
+    Android_Search=0x0086,
+    Android_Back=0x0087,
+    Home_Combo=0x0088,
+    Lock_Combo=0x0089,
+    IOS_Virtual_Keyboard=0x008A,
+    IOS_Language_Switch=0x008B,
+    Mac_Expose=0x008C,
+    Mac_Dashboard=0x008D,
+    Win7_Snap_Left=0x008E,
+    Win7_Snap_Right=0x008F,
     Minimize_Window=0x0090,  # WIN7_MINIMIZE_AS_WIN_ARROW
     Maximize_Window=0x0091,  # WIN7_MAXIMIZE_AS_WIN_ARROW
-    WIN7_STRETCH_UP=0x0092,
-    WIN7_MONITOR_SWITCH_AS_WIN_SHIFT_LEFTARROW=0x0093,
-    WIN7_MONITOR_SWITCH_AS_WIN_SHIFT_RIGHTARROW=0x0094,
+    Win7_Stretch_Up=0x0092,
+    Win7_Monitor_Switch_As_Win_Shift_LeftArrow=0x0093,
+    Win7_Monitor_Switch_As_Win_Shift_RightArrow=0x0094,
     Switch_Screen=0x0095,  # WIN7_SHOW_PRESENTATION_MODE
-    WIN7_SHOW_MOBILITY_CENTER=0x0096,
-    ANALOG_HSCROLL=0x0097,
-    METRO_APPSWITCH=0x009F,
-    METRO_APPBAR=0x00A0,
-    METRO_CHARMS=0x00A1,
-    CALC_VKEYBOARD=0x00A2,
-    METRO_SEARCH=0x00A3,
-    COMBO_SLEEP=0x00A4,
-    METRO_SHARE=0x00A5,
-    METRO_SETTINGS=0x00A6,
-    METRO_DEVICES=0x00A7,
-    METRO_START_SCREEN=0x00A9,
-    ZOOMIN=0x00AA,
-    ZOOMOUT=0x00AB,
-    BACK_HSCROLL=0x00AC,
-    SHOW_DESKTOP_HPP=0x00AE,
+    Win7_Show_Mobility_Center=0x0096,
+    Analog_HScroll=0x0097,
+    Metro_Appswitch=0x009F,
+    Metro_Appbar=0x00A0,
+    Metro_Charms=0x00A1,
+    Calc_Vkeyboard=0x00A2,
+    Metro_Search=0x00A3,
+    Combo_Sleep=0x00A4,
+    Metro_Share=0x00A5,
+    Metro_Settings=0x00A6,
+    Metro_Devices=0x00A7,
+    Metro_Start_Screen=0x00A9,
+    Zoomin=0x00AA,
+    Zoomout=0x00AB,
+    Back_Hscroll=0x00AC,
+    Show_Desktop_HPP=0x00AE,
     Fn_Left_Click=0x00B7,  # from K400 Plus
     # https://docs.google.com/document/u/0/d/1YvXICgSe8BcBAuMr4Xu_TutvAxaa-RnGfyPFWBWzhkc/export?format=docx
     # Extract to csv.  Eliminate extra linefeeds and spaces.
@@ -197,11 +196,11 @@ CONTROL = _NamedInts(
     # gsub(" \\+","_Up",$2); gsub("[()\"-]","",$2); gsub(" ","_",$2); printf("\t%s=0x%04X,\n", $2, $1)}' < controls.cvs
     Second_Left_Click=0x00B8,  # Second_LClick / on K400 Plus
     Fn_Second_Left_Click=0x00B9,  # Fn_Second_LClick
-    MultiPlatform_App_Switch=0x00BA,
-    MultiPlatform_Home=0x00BB,
-    MultiPlatform_Menu=0x00BC,
-    MultiPlatform_Back=0x00BD,
-    MultiPlatform_Insert=0x00BE,
+    Multiplatform_App_Switch=0x00BA,
+    Multiplatform_Home=0x00BB,
+    Multiplatform_Menu=0x00BC,
+    Multiplatform_Back=0x00BD,
+    Multiplatform_Insert=0x00BE,
     Screen_Capture__Print_Screen=0x00BF,  # on Craft Keyboard
     Fn_Down=0x00C0,
     Fn_Up=0x00C1,
@@ -212,26 +211,26 @@ CONTROL = _NamedInts(
     Wifi=0x00C6,
     Brightness_Down=0x00C7,
     Brightness_Up=0x00C8,
-    Display_out__project_screen_=0x00C9,
+    Display_Out__Project_Screen_=0x00C9,
     View_Open_Apps=0x00CA,
-    View_all_apps=0x00CB,
+    View_All_Apps=0x00CB,
     Switch_App=0x00CC,
-    Fn_inversion_change=0x00CD,
-    MultiPlatform_back=0x00CE,
-    Multiplatform_forward=0x00CF,
-    Multiplatform_gesture_button=0x00D0,
-    Host_Switch_channel_1=0x00D1,
-    Host_Switch_channel_2=0x00D2,
-    Host_Switch_channel_3=0x00D3,
-    Multiplatform_search=0x00D4,
-    Multiplatform_Home__Mission_Control=0x00D5,
-    Multiplatform_Menu__Show__Hide_Virtual_Keyboard__Launchpad=0x00D6,
+    Fn_Inversion_Change=0x00CD,
+    MultiPlatform_Back=0x00CE,
+    MultiPlatform_Forward=0x00CF,
+    MultiPlatform_Gesture_Button=0x00D0,
+    Host_Switch_Channel_1=0x00D1,
+    Host_Switch_Channel_2=0x00D2,
+    Host_Switch_Channel_3=0x00D3,
+    MultiPlatform_Search=0x00D4,
+    MultiPlatform_Home__Mission_Control=0x00D5,
+    MultiPlatform_Menu__Show__Hide_Virtual_Keyboard__Launchpad=0x00D6,
     Virtual_Gesture_Button=0x00D7,
-    Cursor_Button_Long_press=0x00D8,
+    Cursor_Button_Long_Press=0x00D8,
     Next_Button_Shortpress=0x00D9,  # Next_Button
-    Next_Button_Longpress=0x00DA,
-    Back_Button_Shortpress=0x00DB,  # Back
-    Back_Button_Longpress=0x00DC,
+    Next_Button_Long_Press=0x00DA,
+    Back_Button_Short_Press=0x00DB,  # Back
+    Back_Button_Long_Press=0x00DC,
     Multi_Platform_Language_Switch=0x00DD,
     F_Lock=0x00DE,
     Switch_Highlight=0x00DF,
@@ -262,16 +261,20 @@ CONTROL = _NamedInts(
     Increase_Color_Effect_Speed=0x00F8,
     Decrease_Color_Effect_Speed=0x00F9,
     Load_Lighting_Custom_Profile=0x00FA,
-    Laser_button_short_press=0x00FB,
-    Laser_button_long_press=0x00FC,
+    Laser_Button_Short_Press=0x00FB,
+    Laser_Button_Long_Press=0x00FC,
     DPI_Switch=0x00FD,
-    MultiPlatform_Home__Show_Desktop=0x00FE,
-    MultiPlatform_App_Switch__Show_Dashboard=0x00FF,
-    MultiPlatform_App_Switch_2=0x0100,  # MultiPlatform_App_Switch
+    Multiplatform_Home__Show_Desktop=0x00FE,
+    Multiplatform_App_Switch__Show_Dashboard=0x00FF,
+    Multiplatform_App_Switch_2=0x0100,  # Multiplatform_App_Switch
     Fn_Inversion__Hot_Key=0x0101,
     LeftAndRightClick=0x0102,
-    LED_TOGGLE=0x013B,  #
+    LED_Toggle=0x013B,  #
 )
+
+for i in range(1, 7):  # add in G keys - these are not really Logitech Controls
+    CONTROL[0x1000 + i] = 'G' + str(i)
+
 CONTROL._fallback = lambda x: 'unknown:%04X' % x
 
 # <tasks.xml awk -F\" '/<Task /{gsub(/ /, "_", $6); printf("\t%s=0x%04X,\n", $6, $4)}'
